@@ -1,4 +1,4 @@
-# Copyright 2019-2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright Amazon.com Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -23,9 +23,10 @@ def validate_circuit_and_shots(circuit: Circuit, shots: int) -> None:
         shots (int): shots to validate
 
     Raises:
-        ValueError: If circuit has no instructions. Also, if no result types
-            specified for circuit and `shots=0`. See `braket.circuit.result_types`.
-            Or, if `StateVector` or `Amplitude` are specified as result types when `shots > 0`.
+        ValueError: If circuit has no instructions; if no result types
+            specified for circuit and `shots=0`. See `braket.circuit.result_types`;
+            if circuit has observables that cannot be simultaneously measured and `shots>0`;
+            or, if `StateVector` or `Amplitude` are specified as result types when `shots>0`.
     """
     if not circuit.instructions:
         raise ValueError("Circuit must have instructions to run on a device")
@@ -34,6 +35,8 @@ def validate_circuit_and_shots(circuit: Circuit, shots: int) -> None:
             "No result types specified for circuit and shots=0. See `braket.circuit.result_types`"
         )
     elif shots and circuit.result_types:
+        if not circuit.observables_simultaneously_measurable:
+            raise ValueError("Observables cannot be sampled simultaneously")
         for rt in circuit.result_types:
             if isinstance(rt, ResultType.StateVector) or isinstance(rt, ResultType.Amplitude):
                 raise ValueError("StateVector or Amplitude cannot be specified when shots>0")
